'use client';

import { FC, useEffect, useState } from 'react';
import type { StripeError } from '@stripe/stripe-js';
import {
    useStripe,
    useElements,
    PaymentElement,
    Elements,
} from '@stripe/react-stripe-js';
import { Form, Button } from 'antd';
import getStripe from '../../utils/get-stripe';
import { createPaymentIntent} from '../../actions/stripe';
import { Payment } from '../../types/payment';
import { i18nText } from '../../i18nKeys';

type PaymentFormProps = {
    amount: number,
    sessionId?: string,
    locale: string
}

export const CheckoutForm: FC<PaymentFormProps> = ({ amount, sessionId, locale }) => {
    const [form] = Form.useForm<Payment>();
    const formAmount = Form.useWatch('amount', form);
    const [paymentType, setPaymentType] = useState<string>('');
    const [payment, setPayment] = useState<{
        status: "initial" | "processing" | "error";
    }>({ status: "initial" });
    const [errorMessage, setErrorMessage] = useState<string>('');
    const stripe = useStripe();
    const elements = useElements();

    const PaymentStatus = ({ status }: { status: string }) => {
        switch (status) {
            case "processing":
            case "requires_payment_method":
            case "requires_confirmation":
                return <h2>Processing...</h2>;

            case "requires_action":
                return <h2>Authenticating...</h2>;

            case "succeeded":
                return <h2>Payment Succeeded</h2>;

            case "error":
                console.log('errorMessage', errorMessage);
                return null;

            default:
                return null;
        }
    };

    useEffect(() => {
        elements?.update({ amount: formAmount * 100 });
    }, [formAmount]);

    const onSubmit = async () => {
        try {
            if (!elements || !stripe) return;

            setPayment({ status: "processing" });

            const { error: submitError } = await elements.submit();

            if (submitError) {
                setPayment({ status: "error" });
                setErrorMessage(submitError.message ?? "An unknown error occurred");

                return;
            }

            const { client_secret: clientSecret } = await createPaymentIntent(
                { amount, sessionId }
            );

            const { error: confirmError } = await stripe!.confirmPayment({
                elements,
                clientSecret,
                confirmParams: {
                    return_url: window.location.href,
                    payment_method_data: {
                        allow_redisplay: 'limited',
                        // billing_details: {
                        //     name: input.cardholderName,
                        // },
                    },
                },
            });

            if (confirmError) {
                setPayment({ status: "error" });
                setErrorMessage(confirmError.message ?? "An unknown error occurred");
            }
        } catch (err) {
            const { message } = err as StripeError;

            setPayment({ status: "error" });
            setErrorMessage(message ?? "An unknown error occurred");
        }
    };

    return (
        <Form form={form} onFinish={onSubmit} style={{ display: 'flex', overflow: 'hidden', flexDirection: 'column', gap: 16, justifyContent: 'space-between', alignItems: 'center' }}>
            <div style={{ width: '100%' }}>
                <PaymentElement
                    onChange={(e) => {
                        setPaymentType(e.value.type);
                    }}
                />
            </div>
            <div>
                <PaymentStatus status={payment.status}/>
            </div>
            <Button
                className="btn-apply"
                htmlType="submit"
                disabled={
                    !["initial", "succeeded", "error"].includes(payment.status) ||
                    !stripe
                }
            >
                {`${i18nText('pay', locale)} ${amount}€`}
            </Button>
        </Form>
    );
}

export const StripeElementsForm: FC<PaymentFormProps> = ({ amount, sessionId, locale }) => {
    return (
        <Elements
            stripe={getStripe()}
            options={{
                fonts: [{
                    cssSrc: 'https://fonts.googleapis.com/css2?family=Comfortaa&display=swap',
                }],
                appearance: {
                    variables: {
                        colorIcon: '#2c7873',
                        fontSizeBase: '16px',
                        colorPrimary: '#66A5AD',
                        colorBackground: '#F8F8F7',
                        colorText: '#000',
                        colorDanger: '#D93E5C',
                        focusBoxShadow: 'none',
                        borderRadius: '8px'
                    },
                },
                currency: 'eur',
                mode: "payment",
                amount: amount*100,
            }}
        >
            <CheckoutForm amount={amount} sessionId={sessionId} locale={locale} />
        </Elements>
    );
};
